/*
  Babix - An educational little preemptive multitask kernel for Arduino Due.
  Franois Pessaux 04/2015.
  Maxime Ayrault 06/2015.
  This code can be freely distributed.
*/

#include <Arduino.h>
#include "mutex.h"
#include "user_tasks.h"


/** Global variable to increment concurrently by several processes. */
volatile uint32_t glob_shared_cpt = 0 ;
/** The lock to protect the shared variable. 0 == free, 1 == taken. */
static mutex_t mutex = 0 ;



/** Simply increment a share global variable, simulating a long processing
   between the readign of the variable and its effective increment.
   The access to the variable is protected by a critical section.
   Basically, this function does:
     read glob_shared_cpt,
     wait,
     glob_shared_cpt = glob_shared_cpt + 1,
     print glob_shared_cpt

   Arg:
    - t : time in ms to wait between reading and writing the shared global
         variable 'glob_shared_cpt'. */
void concurrent_increment (int t)
{
  int tmp ;

  /* Claim the lock to prevent several processes from modifying the kernel
     tables concurrently if they attempt to create processes "at the same
     time". */
  mutex_acquire (&mutex) ;
  tmp = glob_shared_cpt ;
  /* Wait t ms to simulate a looonnnng processing ^^ */
  delay (t) ;
  glob_shared_cpt = tmp + 1 ;
  Serial.println (glob_shared_cpt) ;
  /* Release the mutex to leave the critical section. */
  mutex_release (&mutex) ;
  return ;
}



/** Some examples of user processes. */
void process0 ()
{
  for (;;) {
    Serial.print ("I'm 0 ") ;
    concurrent_increment (15) ;
    delay (70) ;
  }
}

void process1 ()
{
  for (;;) {
    Serial.print("I'm 1 ") ;
    concurrent_increment (99) ;
    delay (470) ;
  }
}

void process2 ()
{
  for (;;) {
    Serial.print ("I'm 2 ") ;
    concurrent_increment (24) ;
    delay (3700) ;
  }
}

void process3 ()
{
  for (;;) {
    digitalWrite (13, HIGH) ;
    delay (1000) ;
    digitalWrite (13, LOW) ;
    delay (1000) ;
  }
}
